#!/usr/local/bin/perl
#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************

# --------------------------------------------------------------
#
# Modification History:
#
# Date    Author Comment
# ======= ====== ===============================================
# 06Oct03 BB     Added debugging code at the end
# 25Apr04 BB     1.13 Added code to capture the rsh service type
# 1.15 14May04 bb - New NTRESKIT code. Now, try to find it in the default pat
# 1.16 05Aug04 bb - New code for 9i connection
# 1.17 29Oct04 bb - New code to find the listener password
# 1.18 17Jan05 bb - New perf subroutine, and much syntax cleanup 
# 1.19 26Jan05 bb - Added spfile logic
# 2.00 15Sep12 BB - Many changes - dusting off 
# 2.01 18Oct12 BB - Debug ORA-12631 problem for COS 
# --------------------------------------------------------------
$version = "2.01";
$P = "dbamon_orant.pl";
$| = 1;

use Win32::TieRegistry;
use Win32::OLE('in');
use Time::Local;

$debug = 0;

#---- Compile warning
$endc = ""; 
%disk_used_mb = (); 

$os   = $ARGV[0];
$verb = $ARGV[1];
$arg1 = $ARGV[2];
$arg2 = $ARGV[3];
$arg3 = $ARGV[4];
$arg4 = $ARGV[5];

$WWW = 1 if $arg1 eq "WWW";

# --------------------------------------------------------------
#
# Subroutines
#
# --------------------------------------------------------------

# ----------------------------------------------------------------------
sub DCFRead 
# ----------------------------------------------------------------------
{
  #---- DCF (DBAmon control file) path
  $dcf_path = "dbamon_orant_dcf_${os}.txt";

  $dcf_debug = 1; 

  &msg("DCFRead - Starting - Reading dcf_path=$dcf_path dcf_debug=$dcf_debug"); 

  if (-e "$dcf_path") 
  {
    #---- Only require the file if it passes a perl syntax check
    ## if (system("perl -wc $dcf_path 2>&1") != 0)
    #---- 2012/07/30 BB - We only want to run this compile check IF we have the path to get to PERL (not always true - UKS2) 
    &msg("Locating PERL and Syntax checking $dcf_path"); 
    if (system("perl -v") == 0 && system("perl -wc $dcf_path 2>&1") != 0)
    {
      &msg("Require file $dcf_path Perl compile error - Not requiring"); 
      # system("echp ' ' | mailx -s 'DBAmon | $host/$os $dcf_path Compile Error' dbaadmin\@dba.it.mycompany.com") if $company eq "mycompany";
    }
    elsif (-z $dcf_path)
    {
      &msg("Require file $dcf_path Is zero bytes - Not requiring"); 
    }
    else
    {
      require "$dcf_path"; 
    }
  }
  else
  {
    &msg("Controlfile file $dcf_path missing - DCFFILEMISSING"); 
    # system("echo ' ' | mailx -s 'DBAmon | $host/$os - DCF $dcf_path Missing' dbaadmin\@dba.it.mycompany.com") if $company eq "mycompany";
  }

  #---- TEMP DEBUG: show variable values
  if ($dcf_debug) 
  {
    &msg("DEBUG - DCF Values read:"); 
    foreach $dcf_key (sort keys %dcf_data)
    {
      &msg("  dcf_data{$dcf_key}=$dcf_data{$dcf_key}"); 
    }
  }

  if ($dcf_data{"LastCycleFinishNum"} > 0)
  {
    $zmins_since = sprintf("%.2f", (time - $dcf_data{"LastCycleFinishNum"}) / 60);
    &msg("DCFRead - Done - Minutes since completion of last iteration: $zmins_since"); 
  }
}

# ----------------------------------------------------------------------
sub DCFWrite 
# ----------------------------------------------------------------------
{
  &msg("DCFWrite - Starting - Writing dcf_path=$dcf_path"); 
  $dcf_debug = 1; 

  #---- Cycle finish variable
  my $z = time; 
  $dcf_data{"LastCycleFinishNum"}  = $z; 
  $dcf_data{"LastCycleFinishChar"} = &timeToText($z); 

  #---- Write out values
  foreach $dcf_key (sort keys %dcf_data)
  {
    $dcf_out .= "\$dcf_data{$dcf_key} = '$dcf_data{$dcf_key}';\n";
    $dcf_count++; 
  }
  ##&msg("DEBUG - dcf_count=$dcf_count dcf_out=$dcf_out"); 

  open(DCF, ">$dcf_path") || &msg("Cannot open output $dcf_path - $!");
  print DCF $dcf_out; 
  close(DCF);

  system("type $dcf_path") if $dcf_debug;

  &msg("DCFWrite - Done - Wrote $dcf_count controlfile entries"); 
}

#------------------------------------------------------------------------------
sub timeToText
#------------------------------------------------------------------------------
{
  $zz_ts = $_[0];
  ($sec,$min,$hour,$mday,$mon,$year) = localtime($zz_ts);
  $year += 1900;
  $mon += 1;

  # Pad with 0's
  $yy = sprintf("%04d",$year);
  $mm = sprintf("%02d",$mon);
  $dd = sprintf("%02d",$mday);
  $hh = sprintf("%02d",$hour);
  $mi = sprintf("%02d",$min);
  $ss = sprintf("%02d",$sec);

  $date = "$yy/$mm/$dd-$hh:$mi:$ss";
  return $date;
}

#------------------------------------------------------------------------------
sub timeToNum
#------------------------------------------------------------------------------
{
  $zz_text = $_[0];
  $zz_text =~ s/-/\//g;
  $zz_text =~ s/:/\//g;
  ##&debug "zz_text=$zz_text\n";
  @words = split(/\//,$zz_text);

  if ($#words != 5)
  {
    &error("Input Target_End_Time ($zz_text) should be in the format: YYYY/MM/DD-HH:MM:SS","DOC116");
  }

  $zz_yyyy = $words[0];
  $zz_mon  = $words[1];
  $zz_dd   = $words[2];
  $zz_hh   = $words[3];
  $zz_mm   = $words[4];
  $zz_ss   = $words[5];
  ##&debug "zz_yyyy=$zz_yyyy zz_mon=$zz_mon zz_dd=$zz_dd zz_hh=$zz_hh zz_mm=$zz_mm zz_ss=$zz_ss\n";

  &error("Input Target_End_Time - YYYY must be 2002-2099","DOC116") if ($zz_yyyy < 2002 || $zz_yyyy > 2099);
  &error("Input Target_End_Time - MON  must be 1-12     ","DOC116") if ($zz_mon  <    1 || $zz_mon  >   12);
  &error("Input Target_End_Time - DD   must be 1-31     ","DOC116") if ($zz_dd   <    1 || $zz_dd   >   31);
  &error("Input Target_End_Time - HH   must be 0-23     ","DOC116") if ($zz_hh   <    0 || $zz_hh   >   23);
  &error("Input Target_End_Time - MM   must be 0-59     ","DOC116") if ($zz_mm   <    0 || $zz_mm   >   59);
  &error("Input Target_End_Time - SS   must be 0-59     ","DOC116") if ($zz_ss   <    0 || $zz_ss   >   59);

  @zz_date=($zz_ss,$zz_mm,$zz_hh,$zz_dd,--$zz_mon,$zz_yyyy);
  $zz_result = &timelocal(@zz_date);

  return $zz_result;
}

#------------------------------------------------------------------------------
sub dbError 
#------------------------------------------------------------------------------
{
  #unused# local($errorMsg,@error_fields) = @_;
  local($errorMsg) = @_;
  &msg( "======== D B   E r r o r   E n c o u n t e r e d =========");
  &msg( "Error: DB Error Encountered While: $errorMsg");
  &msg( "Error: DB Error String: $DBI::errstr");
  &msg( "==========================================================");
  $dberror = 1;

  if ($db_user ne "") 
  {
    &msg("Partial Dump: z1 a 12 84 $db_user rep2nd 99ss $pass yu23 nowisthet1me");
  }

  &msg("END_OF_PROBE");
  exit 1;
}

#------------------------------------------------------------------------------
sub error
#------------------------------------------------------------------------------
{
  my $emsg = $_[0];
  &msg("---------------------------------------------------------------");
  &msg("Fatal Error: $emsg");
  &msg("---------------------------------------------------------------");
  &msg("Exiting RC=1");
  &msg("END_OF_PROBE");
  exit 1;
}

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  my $msgmsg = $_[0];
  $date = &get_date("Y/M/D-H:M:S");
  print "$date ${P} $version | $msgmsg\n";
}

#------------------------------------------------------------------------------
sub debug
#------------------------------------------------------------------------------
{
  my $msgmsg = $_[0];
  return if ! $debug; 
  $date = &get_date("Y/M/D-H:M:S");
  print "$date ${P} $version | **Debug**: $msgmsg\n";
}

#------------------------------------------------------------------------------
sub get_date
#------------------------------------------------------------------------------
{
  # Return date in format supplied as argument. Valid formats are:
  #   Y/M/D-H:M:S
  #   YMD_HMS
  #   YMD

  $format = $_[0];

  @sysdate = localtime;
  for ($i = 0; $i <= 5; $i++)
  {
    if ($sysdate[$i] < 10)
    {
      $sysdate[$i] = 0 . $sysdate[$i];
    }
  }

  if ($format eq "Y/M/D-H:M:S")
  {
    $sysdate = $sysdate[5]+1900 . "/" . ++$sysdate[4] . "/" . $sysdate[3] . "-" . $sysdate[2] . ":" . $sysdate[1] . ":" . $sysdate[0];
  }
  elsif ($format eq "YMD_HMS")
  {
    $sysdate = $sysdate[5]+1900 . ++$sysdate[4] . $sysdate[3] . "_" . $sysdate[2] . $sysdate[1] . $sysdate[0];
  }
  elsif ($format eq "YMD")
  {
    $sysdate = $sysdate[5]+1900 . ++$sysdate[4] . $sysdate[3];
  }
  else
  {
    &error("Invalid format passed to get_date. Must be Y/M/D-H:M:S or YMD_HMS");
  }
  return $sysdate;
}

# ----------------------------------------------------------------------
sub connectDB
# ----------------------------------------------------------------------
{
  &msg("connectDB - Starting");
  use DBI;

  if ($dbh ne "") 
  {
    &msg("  Already connected to DB - Returning"); 
    return; 
  }

  &msg("Connecting to Oracle driver=$driver");
  &msg("(Not using OS authentication") if $db_user ne ""; 

  $dbh = DBI->connect("DBI:ODBC:$driver","$db_user","$pass") || &dbError("Connect_To_DB");

  &msg("connectDB - Done");
}

# ----------------------------------------------------------------------
sub datafiles
# ----------------------------------------------------------------------
{
  &msg("datafiles - Starting");
  use DBI;

  &connectDB; 

  #---- Read from v$datafile;
  $sql = "select 
      d.name, 
      t.name,
      d.status
    from 
      v\$datafile d,
      v\$tablespace t
    where 
      d.ts# = t.ts# and
      status != 'SYSTEM' and 
      status != 'ONLINE'";
  $sth = $dbh->prepare("$sql") || &dbError("Prepare");
  $sth->execute || &dbError("Execute_Select");
  while (@Data = $sth->fetchrow_array)
  {
    $dfname = $Data[0];
    $tsname = $Data[1];
    $status = $Data[2];
    &dataReturn("OfflineDatafile: Tablespace=$tsname DFName=$dfname DFStatus=$status");
  }
  &msg("datafiles - Done");
}

# ----------------------------------------------------------------------
sub ts      
# ----------------------------------------------------------------------
{
  &msg("ts - Starting");

  $t_warn = $arg1;
  $t_crit = $arg2;
  $t_free = $arg3;
  #unused#$t_cmd  = $arg4;

  &connectDB; 

  if (!$dberror)
  {
    $sql = "select 
          a.tablespace_name as tablespace_name,
          d.contents,
          a.sum_bytes as bytes_avail,
          a.max_id,
          a.sum_bytes - nvl(c.sum_bytes,0) as bytes_used,
          nvl(c.sum_bytes,0) as bytes_free,
          round((a.sum_bytes - nvl(c.sum_bytes,0.00)) / a.sum_bytes * 100.0, 2) as pct_used,
          d.status 
     from
          ( select
              tablespace_name,
              sum(bytes) sum_bytes,
              count(1) max_id
            from
              dba_data_files
            group by
              tablespace_name
          ) a,
          ( select
              tablespace_name,
              sum(bytes) sum_bytes
            from
              dba_free_space
            group by
              tablespace_name
          ) c,
          dba_tablespaces d
    where
          a.tablespace_name = c.tablespace_name (+)  
      and a.tablespace_name = d.tablespace_name (+) order by 2;
    ";    

    #&debug("Preparing sql=$sql"); 
    $sth = $dbh->prepare("$sql") || &dbError("Prepare");
    #&debug("Executing sql=$sql"); 
    $sth->execute || &dbError("Execute_Select");
   
    $verb = "Stack: orantTSStack";

    while (@Data = $sth->fetchrow_array)
    {
      $ts   = $Data[0];
      $type = substr($Data[1],0,4);
      $size = $Data[2]/1048576;
      $max  = $Data[3];
      $used = $Data[4]/1048576;
      $free = $Data[5]/1048576;
      $pct  = $Data[6];
      #unused#$stat = $Data[7];

      $sth2 = $dbh->prepare("select max(autoextensible) from dba_data_files where tablespace_name = '$ts'") || &dbError("Prepare");
      $sth2->execute || &dbError("Execute_Select");
      while (@Data2 = $sth2->fetchrow_array)
      {
	$autoextend = $Data2[0];
      }

      ##&debug "ts=$ts autoextend=$autoextend \n";

      $flag = "";
      # MB Needed to get to 5% below warning threshold
      $needed = int($used / (($t_warn-5)/100) - $size);
      if ($needed < 100)
      {
        $needed = 50;
      }
      if ($needed > 2000)
      {
        $needed = 2000;
      }
      
      if ($pct  >= $t_warn && $pct < $t_crit) { $flag = "W" . " " . $needed; }
      if ($pct  >= $t_crit )                  { $flag = "C" . " " . $needed; }
      if ($free >= $t_free)                   { $flag = "";  }
      if ($autoextend eq "YES") { $flag = "A"; } 

      ##&debug "ts=$ts pct=$pct t_warn=$t_warn t_crit=$t_crit flag=$flag \n";

      #&debug("%s %-16s %8d %8d %8d %7.1f %s\n",$verb,$ts,$size,$used,$free,$pct,$flag);
      print "$verb $ts $type $max $size $used $free $pct $flag \n";
    }
    return;
  }

  &msg("ts - Done");
}

# ----------------------------------------------------------------------
sub arclogs
# ----------------------------------------------------------------------
{
  $t_warn = $arg1;
  $t_crit = $arg2;

  &connectDB; 

  if (!$dberror)
  {
    # Read from v$database
    $sql = "select log_mode from v\$database";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare");
    $sth->execute || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      print "Log_Mode: $Data[0]\n";

      if ($Data[0] eq "NOARCHIVELOG")
      {
        &msg("This DB is in NOARCHIVELOG mode - not checking disk");
        return;
      }
    }

    # Read from v$archive_dest
    $sql = "select destination from v\$archive_dest where length(destination) > 0";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare");
    $sth->execute || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      print "Log_Dest: $Data[0]\n";
      $logdrive = uc substr($Data[0],0,1);
      $logdrivedollar = "$logdrive\$";
      print "Log_Drive: $logdrive\n";
      $logdrivecount++;
    }
    print "Log_Drive_Count: $logdrivecount\n";

    #$logdrivesize = (Win32::Driveinfo::DriveSpace($logdrive)) [5];
    #$logdrivefree = (Win32::Driveinfo::DriveSpace($logdrive)) [6];
    #print "size=$logdrivesize free=$logdrivefree \n";

    #---- See how full the $logdrive disk is 
    &debug("logdrive=$logdrive"); 

    open(SRVINFO, "$srvinfo_path -ns|") || print "Error: Running $srvinfo_path";
    while(<SRVINFO>)
    {
      $srvinfo_ran = 1;
      print;
      @words = split;

      if ($words[0] eq $logdrivedollar) 
      {
        $srvinfo_found = 1;
        $logdrivesize = $words[2];
        $logdrivefree = $words[3];
        $logdriveused = $logdrivesize - $logdrivefree;
        $logdrivepct  = int(($logdriveused / $logdrivesize) * 100);
        print "Log_Drive_Size: $logdrivesize\n";
        print "Log_Drive_Free: $logdrivefree\n";
        print "Log_Drive_Used: $logdriveused\n";
        print "Log_Drive_Pct:  $logdrivepct\n";

        if    ($logdrivepct > $t_crit) { $logflag = "C"; }
        elsif ($logdrivepct > $t_warn) { $logflag = "W"; }
        else { $logflag = " "; }
        print "Log_Flag:  $logflag\n";
      }
    }

  } # end of !$dberror

  if (!$srvinfo_ran)
  {
    print "Log_Flag:  S   ($srvinfo_path (srvinfo) did not run)\n";
    system("set");
  }

  if (!$srvinfo_found)
  {
    print "Log_Flag:  S   ($srvinfo_path (srvinfo) did run, but did not find any information for disk $logdrivedollar - something is wrong)\n";
    print "\nNT Environment:\n";
    system("set");
  }
}

# ----------------------------------------------------------------------
sub controlFiles
# ----------------------------------------------------------------------
{
  #---- Read all driveletters from v$controlfile
  &msg("Checking controlfiles - Starting"); 

  $sql = "select name from v\$controlfile where name like '%:%'";
  $sth = $dbh->prepare("$sql") || &dbError("Prepare");
  $sth->execute || &dbError("Execute_Select");

  $cfcount = 0;
  $cfdrives = "";
  $cfdrivecount = 0;

  while (@Data = $sth->fetchrow_array)
  {
    $cfname = $Data[0]; 
    $drive  = substr($cfname,0,1); 
    &msg("  Found cfname=$cfname drive=$drive");
    $cfcount++; 
    $cfdrives .= $drive if index($cfdrives,"$drive") == -1; 
  }

  $cfdrivecount = length($cfdrives); 

  print "$P CFDRIVES: $cfdrives\n"; 
  print "$P CFDRIVECOUNT: $cfdrivecount\n"; 
  print "$P CFCOUNT: $cfcount\n"; 

  &msg("Checking controlfiles - Done"); 
}

# ----------------------------------------------------------------------
sub redoLogs
# ----------------------------------------------------------------------
{
  #---- Read all driveletters from v$logfile
  &msg("Checking redologs - Starting"); 

  $sql = "select member from v\$logfile where member like '%:%'";
  $sth = $dbh->prepare("$sql") || &dbError("Prepare");
  $sth->execute || &dbError("Execute_Select");

  $rlcount = 0;
  $rldrives = "";
  $rldrivecount = 0;

  while (@Data = $sth->fetchrow_array)
  {
    $rlname = $Data[0]; 
    $drive  = substr($rlname,0,1); 
    &msg("  Found rlname=$rlname drive=$drive");
    $rlcount++; 
    $rldrives .= $drive if index($rldrives,"$drive") == -1; 
  }

  $rldrivecount = length($rldrives); 

  print "$P RLDRIVES: $rldrives\n"; 
  print "$P RLDRIVECOUNT: $rldrivecount\n"; 
  print "$P RLCOUNT: $rlcount\n"; 

  &msg("Checking redologs - Done"); 
}

# ----------------------------------------------------------------------
sub up
# ----------------------------------------------------------------------
{
  &msg("up - Starting"); 

  $CurrentVersion = &regRead("HKLM\\Software\\Microsoft\\Windows NT\\CurrentVersion\\CurrentVersion");
  $CSDVersion     = &regRead("HKLM\\Software\\Microsoft\\Windows NT\\CurrentVersion\\CSDVersion");
  ##$win_editon     = &regRead("HKLM\\Software\\Microsoft\\Windows NT\\CurrentVersion\\EditionID");

  if ($CSDVersion ne "")
  {
    @sp_info = split(' ',$CSDVersion);
    $osver = $CurrentVersion . ",SP" . ${sp_info[2]};
  }
  else
  {
    $osver = $CurrentVersion;
  }

  $vendor     = &regRead("HKLM\\Hardware\\Description\\System\\CentralProcessor\\0\\VendorIdentifier");
  $identifier = &regRead("HKLM\\Hardware\\Description\\System\\CentralProcessor\\0\\Identifier");
  $MHz = &regRead("HKLM\\Hardware\\Description\\System\\CentralProcessor\\0\\ProcessorNameString");

  @cpu_ident = split (' ',$identifier);
  $cpumodel = ${cpu_ident[0]} . "/" . ${cpu_ident[2]};
  $cpu = $vendor . " " . " " . $MHz . " MHz ";

  $cpumodel = $ENV{'PROCESSOR_ARCHITECTURE'} . "," . $ENV{'PROCESSOR_LEVEL'};

  if (!$WWW)
  {
    &dataReturn("OSVer: $osver");
    &dataReturn("NumCPUs: $ENV{'NUMBER_OF_PROCESSORS'}");
    &dataReturn("CPU: $cpu");
    &dataReturn("CPUModel: $cpumodel");
  }

  #---- Get memory Info
  print "$P Invoking pstat\n" if !$WWW;
  open(MEM, 'pstat 2>NUL |') || print "$P Error $! Running pstat";
  while(<MEM>)
  {
    ##print;
    @words = split;
    $ram = int($words[4] / 1024);
    last;
  }
  close(MEM);
  if (defined($ram))
  {
    &dataReturn("RAM: $ram");
  }

  &msg("Invoking net start to see which services are running") if !$WWW;

  #---- See if the agent is running
  open(NET, 'net start |') || print "$P Error $! Running net start";

  $rsh_service = "?";

  while(<NET>)
  {
    ##print;
    @words = split;

    if (/TNSListener/)
    {
      $listener_running = 1;
      &dataReturn("ListenerServiceRunning: Y"); 
    }

    if (uc $words[0] eq uc "OracleService$os")
    {
      $service_running = 1;
      &dataReturn("OracleServiceRunning: Y"); 
    }

    if ($rsh_service eq "?")
    {
      if (/Remote Shell Daemon/)  { $rsh_service="RSD"; }
      if (/Remote Shell Service/) { $rsh_service="RSS"; }
      if (/CYGWIN sshd/)          { $rsh_service="CYGWIN"; }
      if (/Openssh SSHD/)         { $rsh_service="COPSSH"; }
    }
  }
  close(NET);

  &msg("After net start") if !$WWW;

  &dataReturn("RSHService: $rsh_service");

  if (!$service_running)
  {
    $dbstatus = "SERVICENOTACTIVE";
    &dataReturn("DBStatus: $dbstatus");
  }

  #---- See if the agent is running

  #---- Determine the name of the lsnrctl program
  if (-x "$oh\\bin\\lsnrctl80.exe")
  {
    $lsnrpgm = "lsnrctl80"; 
  }
  else
  {
    $lsnrpgm = "lsnrctl"; 
  }
  &msg("Running $lsnrpgm") if !$WWW;

  #---- See if the listener.ora file contains a password
  &msg("Looking in $oh\\network\\admin\\listener.ora"); 
  open(LSNR,"$oh\\network\\admin\\listener.ora") || &error("Cannot OPEN listener.ora file");
  while (<LSNR>)
  {
    if (/^PASSWORDS_LISTENER/)
    {
      $listener_password = (split(/\=/,$_))[1];
      $listener_password =~ s/ //g;
      $listener_password =~ s/\(//g;
      $listener_password =~ s/\)//g;
      $listener_password =~ s/\n//g;
    }
  }
  close(LSNR);

  #---- Create temp listener commands file
  $listener_cmds = "\\temp\\dbamon_ora_listener_$os.txt";
  open (LSNRCMDS,">$listener_cmds") || &error("Cannot OPEN output $listener_cmds file");
  print LSNRCMDS "set password $listener_password\n" if $listener_password;
  print LSNRCMDS "status\n"; 
  print LSNRCMDS "exit\n"; 
  close(LSNRCMDS);

  ## &msg("Listener commands");
  ## system("type $listener_cmds");

  #---- New way
  #@lsnr_output = `type $listener_cmds | lsnrctl`;
  #foreach (@lsnr_output)
  #{
  #  print;
  #}
  #---- Old way
  #open(LSNR, "type $listener_cmds | $lsnrpgm |") || print "Error: Running $lsnrpgm status";
  #while(<LSNR>)

  &msg("Running $lsnrpgm with $listener_cmds");
  #---- New way
  @lsnr_output = `type $listener_cmds | $lsnrpgm`;
  #---- Remove temp file
  system("erase $listener_cmds");
  foreach (@lsnr_output)
  {
    print if !$WWW;
    $listener_output .= $_;

    #---- Old way
    #@words = split;
    #if (/command completed successfully/)
    #{
    #  $lsnrctl_running = 1;
    #  #last;
    #}

    #---- New way
    if (/Services Summary.../)
    {
      $lsnrctl_running = 1;
      #last;
    }
  }
  print "\n";

  $lsnr_status = "OK";
  if (!$lsnrctl_running) 
  {
    $lsnr_status = "LSNRCTL-NOT-OK";
  }
  if (!$listener_running) 
  {
    $lsnr_status = "LSNR-SERVICE-NOT-RUNNING";
  }

  &dataReturn("LSNRStatus: $lsnr_status");

  ##&bkup;

  if ($lsnr_status ne "OK")
  {
    &msg("Since the listener is not running properly, aborting further checks");
    if ($WWW) { exit 1; }
    return 1;
  }

  &connectDB; 

  if (!$dberror)
  {
    #---- Read from dual
    &msg("Selecting from dual") if !$WWW;
    $sql = "select dummy from dual";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare");
    $sth->execute || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      $dbdual = $Data[0];
      &dataReturn("DBDual: $dbdual");
    }

    #---- Read from v$instance
    &msg("Selecting from v\$instance") if !$WWW;
    $sql = "select 
      version, 
      status, 
      logins,
      to_char(startup_time,'YYYY/MM/DD-HH24:MI:SS')
    from 
      v\$instance";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare");
    $sth->execute || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      $ver = $Data[0];
      $sta = $Data[1];
      $log = $Data[2];
      $str_c = $Data[3];

      if (!$WWW)
      {
	&dataReturn("DBStatus: $sta");
	&dataReturn("DBVersion: $ver");
	&dataReturn("DBLogins: $log");
      }
    }

    if ($str_c)
    {
      $str_n = &timeToNum($str_c);
      $endc = $str_c;
      $calc_date = $date;
      $calc_time = time;

      $uptime_days  = sprintf("%.2f", (time - $str_n) / (24*60*60));
      $uptime_hours = sprintf("%.2f", (time - $str_n) / (60*60));
      
      if ($WWW ne "Y")
      {
        print "$P UPTIME:          $str_c $str_n $calc_date $calc_time\n";
      }
    }

    #---- Read from v$license
    &msg("Selecting from v\$license") if !$WWW;
    $sql = "select SESSIONS_HIGHWATER from v\$license";
    $sth = $dbh->prepare("$sql") || &dbError("Prepare");
    $sth->execute || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      $hwm = $Data[0];
      &dataReturn("SESSIONSHWM: $hwm");
    }

    #---- Get session userid
    &msg("Selecting from dual for user") if !$WWW;
    $sql = "select user from dual";
    $sth = $dbh->prepare("$sql") 
      || &dbError("Prepare");
    $sth->execute
      || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      $user = $Data[0];
    }
    print "Current_User: $user\n" if !$WWW;

    #---- Get total bytes
    &msg("Selecting from dba_data_files to get size of instance");
    $sql = "select sum(bytes) from dba_data_files";
    $sth = $dbh->prepare("$sql") 
      || &dbError("Prepare");
    $sth->execute
      || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      $tot_spacealloc = int($Data[0] / (1024*1024));
    }

    #---- Get total free
    &msg("Selecting from dba_free_space to get size of free space");
    $sql = "select sum(bytes) from dba_free_space";
    $sth = $dbh->prepare("$sql") 
      || &dbError("Prepare");
    $sth->execute
      || &dbError("Execute_Select");
    while (@Data = $sth->fetchrow_array)
    {
      $tot_spacefree = int($Data[0] / (1024*1024));
    }
    $tot_spaceused = $tot_spacealloc - $tot_spacefree;

    if (!$WWW)
    {
      &dataReturn("SpaceFree:  $tot_spacefree");
      &dataReturn("SpaceAlloc: $tot_spacealloc");
      &dataReturn("SpaceUsed:  $tot_spaceused");
    }
 
  } #---- end of !$dberror

  #---- Decide status
  $dbstatus = "DOWN";

  if ($service_running && $dbdual eq "X" && $sta eq "OPEN")
  {
    $dbstatus = "UP";
  }
  if ($service_running && $log eq "RESTRICTED")
  {
    $dbstatus = "RESTRICTED";
  }
  if ($service_running && $dbdual ne "X")
  {
    $dbstatus = "SQLERROR";
  }

  &dataReturn("DBStatus: $dbstatus");

  if ($WWW)
  {
    $www_bgcolor = "pink";
    $www_bgcolor = "lime" if $dbstatus eq "UP";

    &msg("WWW Status Summary:");
    print "<table border=1 \n";
    print "<tr bgcolor=slate><th><h5>Item<th><h5>Status\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Hostname<td><h5>$host\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>ORACLE_SID<td><h5>$os\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>DB Status<td><h5>$dbstatus\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Listener Status<td><h5>$lsnr_status\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Logins?<td><h5>$log\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Database Space Alloc.<td><h5>$tot_spacealloc mB\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Database Space Used<td><h5>$tot_spaceused mB\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Oracle Version<td><h5>$ver\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>CPU Info<td><h5>$cpu $cpumodel\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>CPU Count<td><h5>$numcpus\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>RAM Size<td><h5>$ram mB\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>OS Version<td><h5>$osver\n";
    ##print "<tr bgcolor=$www_bgcolor><td><h5>Server Up Time<td><h5>$uptime\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Remote Shell Service<td><h5>$rsh_service\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>DB Startup Timestamp:<td><h5>$str_c\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Days DB Up:<td><h5>$uptime_days\n";
    print "<tr bgcolor=$www_bgcolor><td><h5>Hours DB Up:<td><h5>$uptime_hours\n";
    print "</table>\n";

    &msg("Listener Status: $listener_output");

    &msg("Done");
    &msg("END_OF_PROBE");
    #exit 0;
  }

  &msg("up - Done"); 
}

# ----------------------------------------------------------------------
sub bkrun
# ----------------------------------------------------------------------
{
  $cmd = $arg1;
  print "$P bkrun: Issuing at now command to run backup:\n";
  print "$P bkrun: Command before {SID} substitution: $cmd\n";
  $cmd =~ s/\{SID\}/$os/g;
  $cmd = "echo 'time $cmd' | at now";
  print "$P bkrun: About to issue: $cmd\n";
  $rc = system($cmd);
  print "$P bkrun: Back from issuing at now - rc=$rc\n";
}

# ----------------------------------------------------------------------
sub bkup
# ----------------------------------------------------------------------
{
  &msg("bkup - Starting"); 
  
  &connectDB; 

  $sql = "select table_name from dba_tables where owner='DBAMON' and 
          table_name='BACKUP_AGE'";
  $sth = $dbh->prepare("$sql") || &dbError("Prepare");
  $sth->execute || &dbError("Execute_Select");
  while (@Data = $sth->fetchrow_array)
  {
     $bat_exists = "true";   
  }
 
  if ($bat_exists eq "true")
  {
     #---- Read from dbamon.backup_age
     $sql = "select bcklevel, max(tsnum),tstext,elapsed 
       from 
         dbamon.backup_age
       where 
         sid =\'$os\' and bcklevel !=\'ARC\' 
         group by tstext,bcklevel,elapsed
         order by 2,3";
     $sth = $dbh->prepare("$sql") || &dbError("Prepare");
     ##$sth->execute || &dbError("Execute_Select");
     $sth->execute;
     print "=======================================================\n";
     print "Backups Found in dbamon.backup_age:\n\n";

     $lvlage = 99999999;  # Age in hours of most recent LVL? backup
     while (@Data = $sth->fetchrow_array)
     {
       $bcklevel = $Data[0];
       $maxts    = $Data[1];
       $c_date   = $Data[2];
       $e_time   = $Data[3];
       $now = time();
       $agehrs   = ($now - $maxts) / 3600;
       $agehrs   = sprintf("%.0f", $agehrs);
       print "$P BACKUP_AGE: $bcklevel $agehrs - Timestamp=$c_date\n";
       if (substr($bcklevel,0,3) eq "LVL")
       {
          $bktype = "bkRMAN";
          if ($agehrs < $lvlage) { $lvlage = $agehrs; }
       }
       if (substr($bcklevel,0,3) eq "EXP")
       {
          $bktype = "bkEXP";
          if ($agehrs < $lvlage) { $lvlage = $agehrs; }
       }
       if (substr($bcklevel,0,3) eq "TBS")
       {
          $bktype = "bkTBS";
          if ($agehrs < $lvlage) { $lvlage = $agehrs; }
       }
     }
     print "BckAge: $lvlage Type: $bktype CompletionTime: $c_date ElapsedMinutes: $e_time\n"; 
     $flag = 1;
  } 
  else
  {
     #---- Read from v$datafile;
     $sql = "select 
         s.backup_type, 
         s.incremental_level,
         to_char(s.completion_time,'YYYY/MM/DD-HH24:MI:SS'),
         s.elapsed_seconds / 60,
         (sysdate - s.completion_time) * 24
       from 
         v\$backup_set s
       where 
         s.backup_type in ('D','I')
       order by 5 desc
       ";
     $sth = $dbh->prepare("$sql") || &dbError("Prepare");
     $sth->execute || &dbError("Execute_Select");
     print "=======================================================\n\n";
     print "Backups Found in v\$backup_set:\n\n";
     while (@Data = $sth->fetchrow_array)
     {
       $ty = $Data[0];
       $lv = $Data[1];
       $ct = $Data[2];
       $es = int($Data[3]);
       $ag = int($Data[4]);
       print "BckAge: $ag Type: $ty Level: $lv CompletionTime: $ct ElapsedMinutes: $es\n";
       $flag = 1;
     }
   }
  if (!$flag)
  {
    print "BckAge: -1\n";
  }
  &msg("bkup - Starting"); 
}

# ----------------------------------------------------------------------
sub bkhi
# ----------------------------------------------------------------------
{
  &msg("bkhi - Starting - Selecting from DBAMON.BACKUP_AGE");

  &connectDB; 

  #---- Read from dbamon.backup_age
  $sql = "select 
      bcklevel, 
      tstext, 
      tsnum, 
      dbsize, 
      elapsed 
    from 
      dbamon.backup_age
    where 
      sid =\'$os\'
    order 
      by 2 desc";
  $sth = $dbh->prepare("$sql") || &dbError("Prepare");
  $sth->execute;

  $lvlage = 99999999;   #---- Age in hours of most recent LVL? backup
  print "<font face='Trebuchet MS'> <table bgcolor=slate> <tr>
    <th><font size=2>Backup<br>Type
    <th><font size=2>Completion<br>Timestamp
    <th><font size=2>DB Size<br>(MB Used)
    <th><font size=2>Backup Duration<br>(Minutes)
    <th><font size=2>Age Of<br>This Backup<br>(Hours) 
  ";

  $rowcount = 0;

  while (@Data = $sth->fetchrow_array)
  {
    $bcklevel = $Data[0];
    $tstext   = $Data[1];
    $tsnum    = $Data[2];
    $dbsize   = $Data[3];
    $elapsed  = $Data[4];

    $now = time();
    $agehrs   = ($now - $tsnum) / 3600;
    $agehrs   = sprintf("%.0f", $agehrs);

    $elapsed_min = $elapsed / 60;
    $elapsed_min = sprintf("%.2f", $elapsed_min);

    $rowcount++;

    print "<tr bgcolor=lightblue>
      <td><font size=2><b>$bcklevel
      <td><font size=2><b>$tstext
      <td align=right><font size=2><b>$dbsize
      <td align=right><font size=2><b>$elapsed_min
      <td align=right><font size=2><b>$agehrs
    \n";
  }
  print "</table>";
  &msg("Total Backups Found: $rowcount\n");
}

# ----------------------------------------------------------------------
sub perf
# ----------------------------------------------------------------------
{
  &msg("perf - Starting");

  &connectDB; 

  &msg("Running SQL to collect perf data");

  $sql = '
  alter session set NLS_NUMERIC_CHARACTERS = \'.,\';
  select
    \'F\', count(*) from v\$bh where status = \'free\';
  select
    \'P\',
    name,
    100* (1-(physical_reads / (DB_BLOCK_GETS + CONSISTENT_GETS)))
  from
    v\$buffer_pool_statistics;
  select
    \'V\', 
    round((sum(gethits) / sum(gets) * 100),2),
    sum(gets)
     from v\$librarycache;
  select 
    \'U\', 
    round((sum(gets) - sum(getmisses)) / sum(gets) * 100,2),
    sum(gets)
     from v\$rowcache;
  select
    \'R\', bytes from v\$sgastat where name = \'free memory\' and pool = \'shared pool\';
  select
    \'S\', value from v\$parameter where name = \'shared_pool_size\';
  select
    \'T\', value from v\$parameter where name = \'db_block_buffers\';
  select
    \'J\', value from v\$parameter where name = \'db_block_size\';
 select
    \'K\', value from v\$parameter where name = \'spfile\';
  select
    \'Q\', value from v\$parameter where name = \'db_cache_size\';
  select
    \'X\', value from v\$parameter where name = \'sql_trace\';
  select 
    \'W\',
       sum(decode(name, \'consistent gets\', value, 0)) consistent,
       sum(decode(name, \'db block gets\',   value, 0)) db_blockgets,
       sum(decode(name, \'physical reads\',  value, 0)) physical,
       round(((sum(decode(name, \'consistent gets\', value, 0)) +
               sum(decode(name, \'db block gets\',   value, 0)) -
               sum(decode(name, \'physical reads\',  value, 0))) /
              (sum(decode(name, \'consistent gets\', value, 0)) +
               sum(decode(name, \'db block gets\',   value, 0)))) * 100, 2)
    hitratio from v\$sysstat;
  select
    \'Z\',
    100* (1-(sum(physical_reads) / (sum(DB_BLOCK_GETS) + sum(CONSISTENT_GETS))))
  from
    v\$buffer_pool_statistics
  where
    (db_block_gets + consistent_gets) > 0
    and name != \'RECYCLE\';
  ';

  #unused# $sqlgood = 0;
  $free_buffers = -1;

  &oraSQL($sql);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb  = $words[0];
    
    if ($verb eq "F")
    {
      print "F $words[1]\n";
      $free_buffers = $words[1];
    }

    if ($verb eq "R")
    {
      ##print "R $words[1]\n";
      $sg_free = $words[1];
    }

    if ($verb eq "S")
    {
      ##print "S $words[1]\n";
      $sg_size = $words[1];
      if ($sg_size =~ /M$/)
      {
        chop($sg_size);
        $sg_size *= 2**20;
      }
    }

    if ($verb eq "T")
    {
      $db_block_buffers = $words[1];
    }

    if ($verb eq "J")
    {
      $db_block_size = $words[1];
    }

    if ($verb eq "K")
    {
      $spfile = $words[1];
    }

    if ($verb eq "Q")
    {
      $db_cache_size = $words[1];
    }

    if ($verb eq "X")
    {
      $sql_trace = $words[1];
    }

    if ($verb eq "P")
    {
      $pool_keep_hitratio    = $words[2] if $words[1] eq "KEEP";
      $pool_recycle_hitratio = $words[2] if $words[1] eq "RECYCLE";
      $pool_default_hitratio = $words[2] if $words[1] eq "DEFAULT";
    }

    if ($verb eq "W") 
    { 
      $bufcongets   = $words[1];
      $bufblkgets   = $words[2];
      $bufphysreads = $words[3];
      $bufhitratio  = $words[4];
    }

    if ($verb eq "Z") 
    { 
      $bufhitratio_xr   = $words[1];
    }

    if ($verb eq "V") 
    { 
      $libcachehitratio = $words[1];
      $libcachegets     = $words[2];
    }

    if ($verb eq "U") 
    { 
      $dictcachehitratio = $words[1];
      $dictcachegets     = $words[2];
    }
  }
  #unused# close(INDB);

  #Edits
  $pool_default_hitratio = sprintf("%.2f", $pool_default_hitratio) if $pool_default_hitratio ne "";
  $pool_recycle_hitratio = sprintf("%.2f", $pool_recycle_hitratio) if $pool_recycle_hitratio ne "";
  $pool_keep_hitratio    = sprintf("%.2f", $pool_keep_hitratio)    if $pool_keep_hitratio ne "";

  #---- Make sure that something ran above
  &error("SQLPlus Did Not Run") if $sg_size eq ""; 

  if ($sg_size > 0)
  {
    $sg_pct = 100 - (($sg_free / $sg_size) * 100);
  }
  else
  {
    $sg_pct = ".";
  }

  chop($num_otrace = `ls -l \$ORACLE_HOME/otrace/admin/*.dat 2>/dev/null | wc -l`); 

  &dataReturn( "SGA:              $sg_pct $sg_size $sg_free");
  &dataReturn( "POOL_DEFAULT_HR:  $pool_default_hitratio"); 
  &dataReturn( "POOL_RECYCLE_HR:  $pool_recycle_hitratio"); 
  &dataReturn( "POOL_KEEP_HR:     $pool_keep_hitratio"); 
  &dataReturn( "BUFBLKGETS:       $bufblkgets"); 
  &dataReturn( "BUFCONGETS:       $bufcongets"); 
  &dataReturn( "BUFPHYSREADS:     $bufphysreads"); 
  &dataReturn( "BUFHITRATIO:      $bufhitratio"); 
  &dataReturn( "BUFHITRATIO_XR:   $bufhitratio_xr"); 
  &dataReturn( "LIBHITRATIO:      $libcachehitratio"); 
  &dataReturn( "LIBGETS:          $libcachegets"); 
  &dataReturn( "DICTHITRATIO:     $dictcachehitratio"); 
  &dataReturn( "DICTGETS:         $dictcachegets"); 
  &dataReturn( "DB_BLOCK_BUFFERS: $db_block_buffers"); 
  &dataReturn( "DB_BLOCK_SIZE:    $db_block_size"); 
  &dataReturn( "DB_CACHE_SIZE:    $db_cache_size"); 
  &dataReturn( "NUM_OTRACE:       $num_otrace"); 
  &dataReturn( "SQL_TRACE:        $sql_trace"); 
  &dataReturn( "FREE_BUFFERS:     $free_buffers"); 
  &dataReturn( "SPFILE:           $spfile"); 

  #---- Find the names of suitable USER* and TEMP* tablespaces
  &msg("Looking for suitable USER* and TEMP* tablespaces");
  $sql = '
    select
      \'USER_TS\',
      ts.tablespace_name
    from
      dba_tablespaces ts
    where
      ts.tablespace_name like \'USER%\'
    order by
      1 ;

    select
      \'ALT_USER_TS\',
      us.default_tablespace,
      count(*) 
    from
      dba_users us
    where 
      us.default_tablespace != \'SYSTEM\'
    group by
      \'ALT_USER_TS\',
      us.default_tablespace
    order by 3 desc
      ;

    select
      \'ALT_TEMP_TS\',
      us.temporary_tablespace,
      count(*) 
    from
      dba_users us
    where 
      us.temporary_tablespace != \'SYSTEM\'
    group by
      \'ALT_TEMP_TS\',
      us.temporary_tablespace
    order by 3 desc
      ;

    select
      \'TEMP_TS\',
      ts.tablespace_name
    from
      dba_tablespaces ts
    where
      ts.tablespace_name like \'TEMP%\'
      and ts.contents = \'TEMPORARY\'
    order by
      1; ';

  &oraSQL($sql);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if ($verb eq "TEMP_TS" && $good_temp_ts eq "")
    {
      $good_temp_ts = $words[1];
      print "good_temp_ts=$good_temp_ts\n";
    }
    if ($verb eq "USER_TS" && $good_users_ts eq "")
    {
      $good_users_ts = $words[1];
      print "good_users_ts=$good_users_ts\n";
    }
    if ($verb eq "ALT_USER_TS" && $alt_users_ts eq "")
    {
      $alt_users_ts = $words[1];
      print "alt_users_ts=$alt_users_ts\n";
    }
    if ($verb eq "ALT_TEMP_TS" && $alt_temp_ts eq "")
    {
      $alt_temp_ts = $words[1];
      print "alt_temp_ts=$alt_temp_ts\n";
    }
  }

  if ($good_users_ts eq "") { $good_users_ts = $alt_users_ts; }
  if ($good_temp_ts eq "")  { $good_temp_ts  = $alt_temp_ts; }

  if ($good_temp_ts eq "" || $good_temp_ts eq "")  
  { 
    system("echo ' ' | mailx -s 'DBAmon | perf() temp=$good_temp_ts users=$good_users_ts alt_users_ts=$alt_users_ts alt_temp_ts=$alt_temp_ts dbaadmin\@dba.it.agilent.com") if $company eq "agilent";
  }

  print "$P GOOD_USERS_TS: $good_users_ts\n";
  print "$P GOOD_TEMP_TS:  $good_temp_ts\n";

  # START - TEMP FIX - there are many DB's where there is a RPT user that keeps 
  #  getting set to temp=tools; Try to fix that. 

  &oraSQL("select 'X' from dba_users where username = 'RPT' and TEMPORARY_TABLESPACE = 'TOOLS';");
  &msg("Looking for RPT user with TOOLS as temporary tablespace");
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X")
    {
      &msg("Altering RPT to set temporary tablespace to TEMP");
      &oraSQL("alter user RPT temporary tablespace TEMP;");
    }
  }

  #---- END   - TEMP FIX - there are many DB's where there is a RPT user that keeps 
  #----   getting set to temp=tools; Try to fix that. 

  &msg("Done collecting user exception data - temp=$good_temp_ts users=$good_users_ts alt_users_ts=$alt_users_ts alt_temp_ts=$alt_temp_ts");

  #---- User exceptions - Fix 
  ## &userException("Fix");

  #---- User exceptions - Report 
  ## &userException("Report");

  #---- Collect ts exception data
  &msg("Running SQL to find tablespaces in exception status");
  $sql = '

    select
      \'TEMP_PERM\',
      ts.tablespace_name
    from
      dba_tablespaces ts
    where
      tablespace_name = \'TEMP\'
      and ts.contents = \'PERMANENT\'
      ;
  ';

  &oraSQL($sql);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if (
        $verb eq "TEMP_PERM" 
       )
    {
      $ts   = $words[1];
      print "$P TS_EXCEPTION: $verb: $ts\n";
    }
  }
  &msg("Done collecting tablespace exception data");

  &perfRollback;

  &msg("perf - Done");
}

# ----------------------------------------------------------------------
sub perfRollback
# ----------------------------------------------------------------------
{
  &msg("Checking rollback segments");
  #---- Check for rollback segment header waits

  $sqlstmt = 'select \'X\', 
    rn.NAME,
    rs.EXTENTS,
    rs.EXTENDS,
    rs.RSSIZE,
    rs.XACTS,
    rs.GETS,
    rs.HWMSIZE,
    rs.WRAPS,
    rs.AVEACTIVE,
    rs.STATUS,
    rs.WAITS,
    rs.optsize
  from 
    v\$rollstat rs, 
    v\$rollname rn 
  where 
    rn.usn=rs.usn;';

  print "Name          Extents  Extends         RSSize   A Xn's      Gets         HWMSize  Wraps   Avg Actv  Status   Waits Optimal\n";
  print "------------ -------- -------- -------------- -------- ---------- -------------- ------  --------- -------- ------ -------\n";

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words     = split();
    $verb      = $words[0];
    $name      = $words[1];
    $extents   = $words[2];
    $extends   = $words[3];
    $rssize    = $words[4];
    $xacts     = $words[5];
    $num_gets  = $words[6];
    $hwmsize   = $words[7];
    $wraps     = $words[8];
    $aveactive = $words[9];
    $status    = $words[10];
    $waits     = $words[11];
    $optimal   = $words[12];
    if ($verb eq "X")
    {
      printf "%-12s %8d %8d %14d %8d %10d %14d %6s %10d %-8s %6d %-8s\n",
        $name,$extents,$extends,$rssize,$xacts,$num_gets,$hwmsize,$wraps,$aveactive,$status,$waits,$optimal;
      $t_waits += $waits;
      $t_gets  += $num_gets;
    }
  }

  $waits_gets_r = 100 * ($t_waits/$t_gets) if $t_gets > 0;
  $waits_gets_r = sprintf("%8.5f",$waits_gets_r);

  print "$P Rollback_Total_Waits:   $t_waits\n";
  print "$P Rollback_Total_Gets:    $t_gets\n";
  print "$P Rollback_Waits_To_Gets: $waits_gets_r%\n";

  if ($waits_gets_r > 1)
  {
    &msg("Because the ratio is > 1, add more rollback segments");
  }
}

#------------------------------------------------------------------------------
sub oraSQL
#------------------------------------------------------------------------------
{
  my $os_sql = $_[0];

  if ($os_sql eq "") { &error("Null SQL passed to oraSQL"); }

  if ($sqlcmd eq "sp")
  {
    if ($db_user eq "") 
    {
      $os_conn = "connect / as sysdba ";
    }
    else
    {
      $os_conn = "connect $db_user/$pass";
    }

    $os_scmd = "sqlplus /NOLOG";
    $os_set  = "set linesize 9999;";
    $os_end  = "/";
  }
  else
  {
    $os_conn = "connect internal ";
    $os_scmd = "svrmgrl";
    $os_set  = "";
    $os_end  = "";
  }
  &msg("Running os_scmd=$os_scmd");
  $os_sql =~ s/\\//g;
  &msg("DEBUG os_sql=$os_sql") if $debug;

  open(SQLOUT,">$tmpsql") || &error("Cannot open output tmpsql=$tmpsql - $!");
  print SQLOUT "$os_conn\n";
  print SQLOUT "$os_set\n";
  print SQLOUT "$os_sql\n";
  print SQLOUT ";\n";
  print SQLOUT "exit\n";
  close(SQLOUT);

  &msg("Running: $os_scmd \@$tmpsql");

  ##system("set"); 
  ##system("$os_scmd \@$tmpsql");

  @oraSQLout = `$os_scmd \@$tmpsql`;
}

#------------------------------------------------------------------------------
sub regRead
#------------------------------------------------------------------------------
{
  use Win32::OLE;

  my $regkey   = $_[0];
  my $regvalue = ""; 
  &debug("regRead - Reading value for $regkey"); 

  $shell    = Win32::OLE->CreateObject("WScript.Shell");
  $regvalue = $shell->RegRead($regkey);

  &debug("regRead - Done - returning regvalue=($regvalue)"); 
  return $regvalue; 
}

#------------------------------------------------------------------------------
sub init
#------------------------------------------------------------------------------
{
  &msg("init - Starting"); 

  #---- Misc Vars. 

  $os_lc = lc $os; 

  %monthmap = ( 
    'Jan' => '01', 
    'Feb' => '02', 
    'Mar' => '03', 
    'Apr' => '04', 
    'May' => '05', 
    'Jun' => '06', 
    'Jul' => '07', 
    'Aug' => '08', 
    'Sep' => '09', 
    'Oct' => '10', 
    'Nov' => '11', 
    'Sec' => '12' 
  ); 

  #---- Some misc. vars. 
  chop($hostname = `hostname`); 
  $host = $hostname;

  $numcpus = $ENV{"NUMBER_OF_PROCESSORS"};

  chop($whoami = `whoami`); 

  #---- Default oracle userid (os authentication)
  $db_user = "internal";
  $pass = "";

  #---- Save Company name
  open(NSLOOKUP, 'ipconfig/all|') || die "Error running ipconfig";
  while(<NSLOOKUP>)
  {
    ##print;
    @words = split;
    $verb1 = $words[0];
    $verb2 = $words[1];
    ##print "verb1=$verb1 verb2=$verb2\n";
    if ($verb1 eq "Host" && $verb2 eq "Name")
    {
      $compwork = $words[$#words]; 
      ##print "compwork=$compwork\n";
    }
  }
  close(NSLOOKUP);

  chop $compwork;
  @words = split(/\./,$compwork);
  $nexttolast = $#words - 1;
  $company = $words[$nexttolast];
  ##print "$P | Company: $company\n";

  $userid = Win32::LoginName();
  $driver = "DBAmonOracle$os";

  if ($os eq "")
  {
    print "$P | Error: No OracleSID supplied\n";
    &msg("END_OF_PROBE");
    exit 1;
  }
  if ($verb eq "")
  {
    print "$P | Error: No verb supplied\n";
    &msg("END_OF_PROBE");
    exit 1;
  }

  $sid_found = "false";

  #---- Get list of sids that exist
  &debug("Reading registry (services) for list of SIDs"); 

  $sid_found = 0; 
  open(NETSTART, "net start |") || &error("Cannot run NET START - $!");
  while(<NETSTART>)
  {
    my @words = split; 
    next unless $words[0] =~ /^Oracle/; 
    $service_name = $words[0]; 
    &msg("  Found Oracle service_name=$service_name"); 
    if ($service_name eq "OracleService$os")
    {
      $sid_found = 1; 
      &msg("    We found the service for our os=$os SID"); 
    }
  }
  close(NETSTART);
  &debug("Resulting sid_found=$sid_found"); 

  if (! $sid_found) 
  {
    &error("Could not find OracleService$os for our ORACLE_SID");
  }

  #---- Find ORACLE_HOME for our ORACLE_SID

  ##$orahome_raw = $shell->RegRead("HKLM\\SYSTEM\\CurrentControlSet\\services\\OracleService$os\\imagepath");
  $orahome_raw = &regRead("HKLM\\SYSTEM\\CurrentControlSet\\services\\OracleService$os\\imagepath");
  &debug("orahome_raw=$orahome_raw"); 

  $orahome = (split(' ',$orahome_raw))[0];
  $orahome =~ s/\\bin\\ORACLE.EXE$//; 
  $oh      = $orahome; 
  &debug("orahome=$orahome oh=$oh"); 

  #---- See if PW is set for this driver 
  $syspw = &regRead("HKLM\\Software\\ODBC\\ODBC.INI\\$driver\\Password");
  &debug("syspw=$syspw"); 

  #---- Capture oraver
  $oraver     = (split(" ",`sqlplus -v`)) [2];
  $oraver_fam = (split('\.',$oraver)) [0];

  &msg("Note: oraver ($oraver) is NG") if $oraver < 1;

  if ($oraver_fam > 8)
  {
    $sqlcmd = "sp";
    $db_user = "";
    $pass    = "/ as sysdba";
    &msg("Note: ORAFAM <= 8"); 
  }
  else
  {
    $sqlcmd = "sv";
    &msg("Note: ORAFAM > 8"); 
  }

  if ($syspw)
  {
    $db_user = "";
    $pass    = "";
    &msg("Note: Set from SYSPW"); 
  }

  #---- Get oradrive
  $oradrive = substr($oh,0,2); 

  $tmpsql = "dbamon_orant_sql_${os}.sql";

  #---- Set ORACLE_SID to $os
  $ENV{'ORACLE_SID'} = $os; 
  $ENV{'ORACLE_HOME'} = $oh; 

  #---- Take these out for now - BB 14May04
  #if ($host eq "abc123") { $ENV{'NTRESKIT'} = "c:\\reskit"; }

  #---- Add $oh/bin to path
  $path_orig   = $ENV{'Path'};
  $path_new    = "${oh}/bin;" . $path_orig;
  $ENV{"Path"} = $path_new;

  #---- OLD: See if there is a pw file - if there is then use that as the pw
  $f = "$oh\\net80\\admin\\.system";
  if (-r "$f")
  {
    $/="\n";
    chomp($pass = `type $f`);
    &msg("Access shortcut1 ON");
  }

  #---- NEW: See if there is a pw file - if there is then use that as the pw
  $f = "$oh\\network\\admin\\.dba_in.txt";
  if (-r "$f")
  {
    open(AL, "$f") || &error("Cannot open input f=$f - $!"); 
    while(<AL>) 
    {
      my @zwords = split;
      if ($zwords[0] eq "User:") { $db_user = $zwords[1]; } 
      if ($zwords[0] eq "PW:")   { $pass    = $zwords[1]; } 
    }
    close(AL);
    &msg("Access shortcut2 ON");
  }

  #---- NEW(er): See if there is a instance-specific pw file - if there is then use that as the pw
  $f = "$oh\\network\\admin\\.dba_in_${os_lc}.txt";
  if (-r "$f")
  {
    open(AL, "$f") || &error("Cannot open input f=$f - $!"); 
    while(<AL>) 
    {
      my @zwords = split;
      if ($zwords[0] eq "User:") { $db_user = $zwords[1]; } 
      if ($zwords[0] eq "PW:")   { $pass    = $zwords[1]; } 
    }
    close(AL);
    &msg("Access shortcut3 ON");
  }

  #---- Display banner
  &msg("  host=$host os=$os oh=$oh userid=$userid whoami=$whoami verb=$verb driver=$driver version=$version company=$company debug=$debug");
  &msg("  arg1=$arg1 arg2=$arg2 arg3=$arg3 arg4=$arg4 WWW=$WWW oraver=$oraver oraver_fam=$oraver_fam sqlcmd=$sqlcmd oradrive=$oradrive srvinfo_path=$srvinfo_path");
  ##&msg("  db_user=$db_user pass=$pass");

  my $z1 = length($db_user); 
  my $z2 = length($pass); 
  &msg("  z1=$z1 z2=$z2");

  &msg("init - Done"); 
}

# ----------------------------------------------------------------------
sub driveCheckWMI
# ----------------------------------------------------------------------
{
  &msg("driveCheckWMI - Loading DRIVE data into hash"); 

  use Win32::OLE('in'); 

  use constant wbemFlagReturnImmediately => 0x10; 
  use constant wbemFlagForwardOnly => 0x20; 

  $computer = ".";
  $objWMIService = Win32::OLE->GetObject
      ("winmgmts:\\\\$computer\\root\\CIMV2");  #--- or die "WMI connection failed.\n";
 
  if (! $objWMIService)
  {
    &msg("objWMIService is NULL - Error=($!) - Running driveCheckCMD instead");
    &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting
    &driveCheckCMD;
    return;
  }

  $colItems = $objWMIService->ExecQuery 
      ("SELECT deviceid, size, freespace, status, statusinfo FROM Win32_LogicalDisk where drivetype = 3","WQL",wbemFlagReturnImmediately | wbemFlagForwardOnly); 
   
  $drive_count = 0;
  $tot_mb_size = 0;
  $tot_mb_free = 0;
  undef %disks;

  &setlocale_C;  #---- Reset locale in case OLE changed it to the local setting

  foreach my $objItem (in $colItems) 
  { 
    #print "Device ID: $objItem->{DeviceID}\n"; 
    #print "Free Space: $objItem->{FreeSpace}\n"; 
    #print "Size: $objItem->{Size}\n"; 

    $drive = $objItem->{DeviceID}; 
    $drive =~ s/\://;
    $disks{"$drive"} = 1;

    $disk_size_mb{"$drive"} = sprintf("%.2f", ($objItem->{Size}      / (1024*1024))  ); 
    $disk_free_mb{"$drive"} = sprintf("%.2f", ($objItem->{FreeSpace} / (1024*1024))  ); 
    $disk_used_mb{"$drive"} = sprintf("%.2f", ($disk_size_mb{"$drive"} - $disk_free_mb{"$drive"}));

    $disk_size_gb{"$drive"} = sprintf("%.2f", ($objItem->{Size}      / (1024*1024*1024))  ); 
    $disk_free_gb{"$drive"} = sprintf("%.2f", ($objItem->{FreeSpace} / (1024*1024*1024))  ); 
    $disk_used_gb{"$drive"} = sprintf("%.2f", ($disk_size_gb{"$drive"} - $disk_free_gb{"$drive"}));

    $disk_pct_full{"$drive"} = sprintf("%.2f", 100 * ($disk_used_gb{"$drive"} / $disk_size_gb{"$drive"})) if  $disk_size_gb{"$drive"} > 0;
    $drive_count++; 

    $tot_mb_size += $disk_size_mb{"$drive"}; 
    $tot_mb_free += $disk_free_mb{"$drive"}; 
  } 

  #---- See if we have ANY results? 
  if ($tot_mb_size == 0) 
  {
    &msg("Even after running WMI, tot_mb_size=$tot_mb_size - We will try driveCheckCMD instead");
    &driveCheckCMD;
    return;
  }

  &msg("  Resulting Disk info:");
  foreach $zdrive (sort keys %disks)
  {
    &msg("    Disk: $zdrive - Size=$disk_size_gb{$zdrive} Free=$disk_free_gb{$zdrive} Used=$disk_used_gb{$zdrive}  PctF=$disk_pct_full{$zdrive}"); 
  }

  &msg("driveCheckWMI - Done - tot_mb_size=$tot_mb_size tot_mb_free=$tot_mb_free"); 
}

# ----------------------------------------------------------------------
sub setlocale_C
# ----------------------------------------------------------------------
{
  &msg("setlocale_C - Starting - Resetting LC_NUMERIC to C (just in case it changed)");
  use POSIX;
  setlocale(LC_NUMERIC, "C");
  &msg("setlocale_C - Done");
}

# ----------------------------------------------------------------------
sub dataReturn
# ----------------------------------------------------------------------
{
  my $in_data = $_[0];
  return if $WWW; 
  print "$P ${in_data}\n";
  return;
}

# ----------------------------------------------------------------------
sub WMI_physicalMemory 
# ----------------------------------------------------------------------
{
  &msg("WMI_physicalMemory - Starting"); 

  use Win32::OLE qw( EVENTS HRESULT in );

  my $machine                         = shift || '.'; # Default to local machine
  my $WMI_MEMORY_CLASS_NAME  = 'Win32_PhysicalMemory';
  my $MEGABYTE               = 1024*1024;

  my $WMI = Win32::OLE->GetObject( "winmgmts:{impersonationLevel=impersonate,(security)}//$machine/" ) || &error("WMI GetObject failed - $!");
  my $total_capacity                  = 0;

  foreach my $object ( in( $WMI->InstancesOf( $WMI_MEMORY_CLASS_NAME ) ) )
  {
    $total_capacity += $object->{Capacity};
  }

  my $total_capacity_in_mb            = $total_capacity / $MEGABYTE;

  &msg("WMI_physicalMemory - Done - Returning total_capacity_in_mb=$total_capacity_in_mb"); 

  return $total_capacity_in_mb;
}

# ----------------------------------------------------------------------
sub alertLog_Filter
# ----------------------------------------------------------------------
{
  my $alf_in = lc $_[0]; 
  my $alf_debug  = 1; 
  my $alf_return = 0; 
  my $alf_included = -1; 
  my $alf_excluded = -1; 

  &msg("  alertLog_Filter - Starting - alf_in=$alf_in") if $alf_debug;  

  #---- See if this line is INCLUDED
  foreach $this_str (@alertchk_include)  
  {
    if (index($alf_in, $this_str) > -1) 
    { 
      $alf_included = 1; 
      &msg("    ALF Included HIT - ($this_str)") if $alf_debug;  
      last; 
    } 
  }
  
  #---- See if this line is EXCLUDED
  foreach $this_str (@alertchk_exclude)  
  {
    if (index($alf_in, $this_str) > -1) 
    { 
      $alf_excluded = 1; 
      &msg("    ALF Excluded HIT - ($this_str)") if $alf_debug;  
      last; 
    } 
  }

  &msg("  After checks - alf_included=$alf_included alf_excluded=$alf_excluded") if $alf_debug;  

  $alf_return = 1 if $alf_included == 1 && $alf_excluded != 1; 
  
  &msg("  alertLog_Filter - Done - returning $alf_return") if $alf_debug;  
  return $alf_return;
}

# ----------------------------------------------------------------------
sub alertLog
# ----------------------------------------------------------------------
{
  &msg("alertLog - Starting - Checking ALERT LOG");
  #---- Check for rollback segment header waits

  $sqlstmt = "select 'X', value from v\$parameter where name = 'background_dump_dest';";

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    my @words = split();
    my $verb = $words[0];  
    if ($verb eq "X")
    {
      $background_dump_dest = $words[1];
    }
  }
  &msg("  background_dump_dest=$background_dump_dest"); 

  my $alertlog_file = "$background_dump_dest\\alert_$os.log"; 
  &msg("  Checking existence of alertlog_file=$alertlog_file"); 
  &error("Cannot find alertlog_file=$alertlog_file") if ! -e "$alertlog_file"; 

  &msg("  Reading alert log file"); 

  $al_lastdate = ""; 
  $al_counter = 0; 
  $al_total   = 0; 
  $al_newrows = 0; 
  @al_rows = (); 
  @al_event_rows = (); 
  $al_events = 0; 

  #---- NEW - From DCF
  $in_rtimestamp = $dcf_data{'AlertLogBookmark'};
  &msg("  (DCF) Input dcf_data{AlertLogBookmark}=$dcf_data{'AlertLogBookmark'} - We will begin processing at in_rtimestamp=$in_rtimestamp");
  &msg("  (DCF) PHONEHOME: Input_DCF_SQLLogBookMark_is_NULL_NORMAL_FOR_FIRST_ITERATION") if ! $dcf_data{'AlertLogBookmark'};
  &msg("  (DCF) PHONEHOME: Input_in_rtimestamp_is_NULL_NORMAL_FOR_FIRST_ITERATION") if ! $in_rtimestamp;

  #---- Common logic
  &msg("  We will begin processing at in_rtimestamp=$in_rtimestamp");

  open(AL, "$alertlog_file") || &error("Cannot open input alertlog_file=$alertlog_file - $!"); 
  while(<AL>) 
  {
    ##print; 
    my $row = $_; 
    $row =~ s/\n//; 
    my @zwords = split; 
    if ($zwords[$#zwords] =~ /^2/ && length($zwords[$#zwords]) == 4) 
    {
      #---- This is a DATE Line
      #----   Sun Sep 16 07:01:36 2012
      #&msg("DEBUG - Found date line - row=$row"); 
      my @zwords2 = split(/\s/, $row); 
      my $zyear = $zwords2[4]; 
      my $zmoni = $zwords2[1];
      my $zmon  = $monthmap{$zmoni};
      my $zday  = $zwords2[2];
      my $ztime = $zwords2[3];

      $al_lastdate = "$zyear/$zmon/$zday-$ztime"; 
      $al_counter  = 0; 
   
      #&msg("DEBUG - al_lastdate=$al_lastdate"); 
      next; 
    }
    $al_counter++; 
    $al_total++; 
    push (@al_rows,  "$al_lastdate [$al_counter] $row");  

    #---- This is where we will check for Alert Log errors/events
    if ($al_lastdate gt $in_rtimestamp) 
    {
      $al_newrows++; 
      if (&alertLog_Filter($row) == 1) 
      {
        push(@al_event_rows, "$al_lastdate [$al_counter] $row"); 
        $al_events++; 
      }
    }
  }
  
  &msg("  Done reading Alert Log - al_events=$al_events al_total=$al_total al_lastdate=$al_lastdate al_newrows=$al_newrows"); 

  &msg("  Alert Log Events (if any):"); 
  foreach $thisrow (@al_event_rows) 
  {
    &dataReturn("AlertLogEvent: $thisrow"); 
  }

  #---- Logic for saving the last NN lines of the Alert Log to the drilldown WWW page
  my $ahowmany = 50; 
  my $astart   = $al_total - $ahowmany;   
  &msg("  $ahowmany Most recent Alert Log rows - astart=$astart"); 

  my $acount = 0; 
  foreach $thisrow (@al_rows) 
  {
    $acount++; 
    ##&msg("acount=$acount astart=$astart"); 
    if ($acount > $astart) 
    {
      &dataReturn("AlertLast: $thisrow"); 
    }
  }

  #---- NEW - Save the last timestamp that we checked
  $dcf_data{'AlertLogBookmark'} = $al_lastdate; 

  &msg("alertLog - Done");
}

# ----------------------------------------------------------------------
sub CFGRead
# ----------------------------------------------------------------------
{
  my $cfg_file  = "dbamon_orant.cfg"; 
  my $cfg_debug = 0; 
  &msg("CFGRead - Starting - cfg_file=$cfg_file"); 

  open(CFG, "$cfg_file") || &error("Cannot open input cfg_file=$cfg_file - $!"); 
  while(<CFG>) 
  {
    next if /^\#/; 

    my @zwords = split; 
    my $row    = $_; 
    my $verb1  = $zwords[0]; 
    my $verb2  = $zwords[1]; 
    my $noun   = lc substr($row,11); 
    
    $noun =~ s/\n//; 

    &msg("Found verb1=$verb1 verb2=$verb2 noun=($noun)") if $cfg_debug; 

    if    ($verb1 eq "CHKALERT" && $verb2 eq "I") 
    {
      push(@alertchk_include, $noun);  
    }
    elsif ($verb1 eq "CHKALERT" && $verb2 eq "E") 
    {
      push(@alertchk_exclude, $noun);  
    }
  }

  #---- Verify CHKALERT arrays
  &msg("  alertchk_include Array Max: $#alertchk_include"); 
  &msg("  alertchk_exclude Array Max: $#alertchk_exclude"); 

  &msg("CFGRead - Done"); 
}

#------------------------------------------------------------------------------
sub PC_PerfBulk
#------------------------------------------------------------------------------
{
  &msg("PC_PerfBulk - Starting");
  $spb_stime = time; 

  chop($whoami = `whoami`); 

  chop($host=`hostname`); 
  $host_uc = uc $host; 

  #---- Each DB seems to have a different way of specifying the first part of the verb 
  #---- So by default we will use "SQLServer:", but we will query sys.dm_os_performance_counters to get the real value

  #$counter_prefix = "SQLServer";   #---- Default

  undef %sysperf_bulk;
  undef %perf_counters;
  my $perf_probeout; 
  $debug_perfbulk = 0; 

  #if ($vershort > 80)
  #{
  #  &connectDB if ! $dbh; 
  #  &msg("Selecting from dm_os_performance_counters to obtain counter_prefix"); 
  #  $sql = "select top 1 OBJECT_NAME from sys.dm_os_performance_counters";
  #  $sth = $dbh->prepare("$sql") || &dbError("Prepare1954");
  #  $sth->execute                || &dbError("Execute_Select1955");
  #  while (@Data = $sth->fetchrow_array)
  #  {
  #    $counter_string = $Data[0];
  #  }
  #  $sth->finish || &dbErrorOK("Finish1960");
  #  $counter_prefix_raw = (split(":",$counter_string)) [0];
  #  $counter_prefix = $counter_prefix_raw if $counter_prefix_raw;
  #  &msg("Resulting counter_string=$counter_string counter_prefix_raw=$counter_prefix_raw"); 
  #}
  #&msg("Using counter_prefix=$counter_prefix"); 

  #---- Now collect the data from the windoze performance counters

  #---- NEW - Collect from parms in CFG file - Write all out to $sps_file
  $sps_file_in  = "dbamon_typeperf_in.txt"; 
  open(SPSFILE,">$sps_file_in") || &error("Cannot open output $sps_file_in - $!"); 

  &msg("Writing Perf Counters from CFG file to $sps_file_in"); 
  foreach (sort keys %cfg_perfcounter)
  {
    $counter     = $_; 
    $dbamon_verb = $cfg_perfcounter{$_};

    ##if ($counter =~ /SQLSERVERPREFIX/) { $counter =~ s/SQLSERVERPREFIX/$counter_prefix/; } 
    
    &msg("  Found: counter=$counter dbamon_verb=$dbamon_verb - Writing to $sps_file_in") if $debug_perfbulk;
    print SPSFILE "$counter\n"; 
    $counter =~ s/^\\//; 
    $perf_counters{$counter} = $dbamon_verb;
  }
  close(SPSFILE); 

  #---- Decide which command to use - typeperf or dbamon_mssql_tpr 
  $typeperf_cmd = "typeperf.exe"; 
  &msg("Verifying that typeperf exists"); 
  open(SPSCMD,"$typeperf_cmd 2>&1 |"); 
  while(<SPSCMD>)
  {
    if (/is not recognized as an internal or external command/i || -e "Override_TPE.txt")
    {
      $typeperf_cmd = "dbamon_mssql_tpe.exe";
      &msg("  typeperf NG - Will use typeperf_cmd=$typeperf_cmd");
      last;
    } 
  }
  close(SPSCMD); 
  &msg("We will be using typeperf_cmd=$typeperf_cmd"); 

  #---- Run typeperf and collect the results
  $sps_first_line = 0; 

  $sps_cmd = "$typeperf_cmd -cf $sps_file_in -sc 1"; 

  &msg("Running sample of performance counters - sps_cmd=$sps_cmd"); 

  open(SPSCMD,"$sps_cmd 2>&1 |"); 
  while(<SPSCMD>)
  {
    next if /^\s$/; 
    next if /^Warning:/; 
    print;
    
    @sps_words = split (/\,/,$_) ; 
    if (! $sps_first_line && /CSV /) 
    {
      $icnt = 0; 
      foreach $this_word (@sps_words)
      {
        $this_word =~ s/\n//g; 
        $this_counter = $this_word; 

        ##print "B4 host_uc=($host_uc) this_counter=$this_counter\n"; 
        $this_counter =~ s/\\\\$host_uc\\//; 
        ##print "AF host_uc=($host_uc) this_counter=$this_counter\n"; 

        $this_counter =~ s/\"//g; 
        $icnt++; 
        $typeperf_keys{$icnt} = $this_counter; 
        &msg("  DEBUG - COUNTER - icnt=$icnt this_word=$this_word this_counter=$this_counter") if $debug_perfbulk; 
      }
      $sps_first_line = 1; 
      next; 
    }
    else   #---- There should just be 2 non-null lines
    {
      $icnt = 0; 
      foreach $this_word (@sps_words)
      {
        next if $this_word =~ /CSV /; 

        $this_word =~ s/\n//g; 
        $this_word =~ s/\"//g; 

        #------------------------------------------------------------------------
        # 2.27 - 2012/04/07 BB: Strange problem found by City of COS. They had
        #   some servers where missing perf counters (not found on that server)
        #   had OK FIELD NAMES but every missing counter showed a -1 in the value
        #   line. So, this line of code will skip over all values that are -1 in 
        #   such a way that we don't increment $icnt, which should make the number
        #   of values equal to the number of column names in the previous line. 
        #   Strange!!!
        # 
        next if $this_word == -1;     #--- Skip bad values and get back in sync
        #------------------------------------------------------------------------
        $icnt++; 
        $this_key         = $typeperf_keys{$icnt}; 
        $this_dbamon_verb = $perf_counters{$this_key}; 
        &msg("  DEBUG - VAL - icnt=$icnt this_word=$this_word this_key=$this_key this_dbamon_verb=$this_dbamon_verb") if $debug_perfbulk; 
        next if ! $this_dbamon_verb; 

        #---- Processing for counters with many decimal places
        if ($this_dbamon_verb =~ /MULT1000$/)
        {
          $this_dbamon_verb =~ s/MULT1000//; 
          $this_word = $this_word * 1000; 
        }
     
        $perf_probeout .= "$P SYSPERF: $this_dbamon_verb $this_word\n"; 
        $sysperf_bulk{$this_dbamon_verb} = $this_word; 
        $sysperf_bulk{$this_dbamon_verb} = sprintf("%.2f", $sysperf_bulk{$this_dbamon_verb}); 
      }
      last; 
    }
  }
  close(SPSCMD);

  #---- We have now collected all of the data

  my $sp_count_nonzero = 0; 
  &msg("Resulting hash sysperf_bulk:"); 
  foreach $this_key (sort keys %sysperf_bulk)
  {
    &msg("  Key=$this_key Value=$sysperf_bulk{$this_key}");
    $sp_count_nonzero++ if $sysperf_bulk{$this_key} > 0;   
  }
  &msg("PC_PerfBulk - Count of non-zero values: $sp_count_nonzero"); 
  &msg("PHONEHOME: PC_PerfBulk - Count_of_non-zero_values=${sp_count_nonzero}_(too_few)") if $sp_count_nonzero < 1; 

  #---- Write out the counters that we collected
  if ($perf_probeout) 
  { 
    &msg("PERF Probeout:\n$perf_probeout"); 
  }
 
  if ($sysperf_bulk{"MemAvailMBytes"} > 0) 
  {
    $mem_avail = $sysperf_bulk{"MemAvailMBytes"} + 0;  
    &msg("Set mem_avail from sysperf_bulk hash - mem_avail=$mem_avail"); 
  } 
  
  $spb_elapsed = time - $spb_stime; 
  &msg("PC_PerfBulk - Done - Elapsed: $spb_elapsed second(s)");
}

# ----------------------------------------------------------------------
sub PC_readCfg
# ----------------------------------------------------------------------
{
  &msg("PC_readCfg - Starting"); 

  $file_cfg_local = "dbamon_orant_local.cfg"; 
  $file_cfg = "dbamon_orant.cfg"; 

  &msg("Checking existence of LOCAL CFG file $file_cfg_local"); 
  if (-e "$file_cfg_local" && -r "$file_cfg_local")
  {
    $file_cfg = $file_cfg_local; 
    &msg("LOCAL CFG File DOES exist - It will be used ($file_cfg)"); 
  }
  else
  {
    &msg("LOCAL CFG File does NOT exist - Default file will be used ($file_cfg)"); 
  }

  &msg("Reading - file_cfg=$file_cfg"); 

  $cnt_cfg = 0;
  $cfg_debug = 1; 

  if (-e "$file_cfg" && -r "$file_cfg")
  {
    open(CFG, "$file_cfg") || &error("Error: Count not open input file_cfg=$file_cfg - $!");
    while(<CFG>)
    {
      ##print;
      next if /^#/ || /^$/;
      $cnt_cfg++; 
      @words = split;

      if ($words[0] eq "PERFCOUNTER" )
      {
	$zp1 = (split(/\|/,$_))[1];
	$zp2 = (split(/\|/,$_))[2];
        $cfg_perfcounter{$zp1} = $zp2; 
      }
    }
    close(CFG); 
  }
  else
  {
    &msg("Config file $file_cfg does not exist or is not readable"); 
  }

  if ($cfg_debug)
  {
    if (! $WWW)
    {
      &msg("Resulting \%cfg_perfcounter"); 
      foreach (sort keys %cfg_perfcounter)
      {
        $zp1 = $_; 
        $zp2 = $cfg_perfcounter{$_};
        print "key=($zp1),val=($zp2)\n";
      }
    }
  }

  &msg("PC_readCfg - Done - Non-comment lines read: $cnt_cfg"); 
}

# ----------------------------------------------------------------------
#
# Main
#
# ----------------------------------------------------------------------

&msg("Starting - os=$os verb=$verb arg1=$arg1 arg2=$arg2"); 
$stime = time;

#---- Read various values from the registry
&init;
&driveCheckWMI;
$ram = &WMI_physicalMemory; 

&DCFRead; 
&CFGRead; 

#---- Check to see if Oracle is 
if ($verb eq "up") 
{
  $subrc = &up();
  $subrc = &ts();
  $subrc = &perf();
  $subrc = &controlFiles();
  $subrc = &redoLogs();
  $subrc = &alertLog();
  #---- PerfCtr Call
  $subrc = &PC_readCfg;
  $subrc = &PC_PerfBulk;

  #---- A few last metrics
  &msg("Final Perf Metrics - ram=$ram mem_avail=$mem_avail");
  if ($ram > 0 && $mem_avail ne "")
  {
    $mem_used = $ram - $mem_avail;
    $mem_pct  = sprintf("%.2f",($mem_used / $ram) * 100);
    &dataReturn("SYSPERF: MemUsedPct $mem_pct");
  }
}

#---- Check to see if the listener is up - start it if not
if ($verb eq "datafiles") 
{
  $subrc = &datafiles();
}

#---- Backup age
if ($verb eq "bkup")
{
  $subrc = &bkup();
}

#---- Archive logs 
if ($verb eq "arclogs")
{
  $subrc = &arclogs();
}

#---- Backup menu options
if ($verb eq "bkhi") { $subrc = &bkhi(); }

#---- Testing
if ($verb eq "perftest") { $subrc = &perf(); }

#---- Terminate

##&msg("Capturing etime");
$etime = time;
$elapsed = $etime - $stime;

&DCFWrite; 

#---- Cleanup
unlink "$tmpsql" if -e "$tmpsql";

&msg("DoneOK - elapsed: $elapsed - subrc: $subrc");
&msg("END_OF_PROBE");
exit 0;
